//############################################################################
//
// EZFB ~ Linux Frame Buffer API ~
//
// by James Lehman
// james@akrobiz.com
//
// began: Feb. 2001
//
// EZFB is intended to give C programmers an easy-to-use library of functions
// for drawing points, lines and strings of characters into the Linux frame
// buffer. It also provides a means of displaying bitmap images on any portion
// of the screen and capturing the contents of any portion of the screen to
// a standard bitmap graphics file. All of this is embodied in a set of
// functions that are polymorphic with regard to the characteristics of the
// frame buffer and the bitmap files used. In other words, it makes no
// difference what the screen height or width is or what the color depth is,
// all of the function calls are the same and will have appropriate and
// predictable effects.
//
// This software is distributed in the hope that it will be useful, but
// without any warranty; without even the implied warranty of merchantability
// or fitness for a particular purpose. See the GNU General Public License
// for more details.
//
//############################################################################
#ifdef __cplusplus
extern "C"
{
#endif        /* __cplusplus */

#ifndef __bmp_FUNCTIONS__
#define __bmp_FUNCTIONS__

//############################################################################
#include "ezfb_types.h"

//############################################################################
struct ezfb_bitmap
{                                //  defaults     offset
    char       B               ; //  'B'          0 :
    char       M               ; //  'M'          1 :
    u_int      file_size       ; //               2 :
    int        reserved        ; //  0            6 :
    int        jump            ; //              10 : offset to pixel data
    int        to_end          ; //  40          14 : from here to end of header
    u_int      xres            ; //              18 :
    u_int      yres            ; //              22 :
    u_short    planes          ; //  1           26 :
    u_short    bpp             ; //              28 : bits per pixel
    int        comp            ; //  0           30 : no compression
    u_int      image_size      ; //              34 : in bytes
    u_int      xppm            ; //  2835        38 : pixels per meter
    u_int      yppm            ; //  2835        42 : approx. 72 dots per inch
    u_int      c_used          ; //              46 :
    u_int      c_important     ; //              50 : ................
    u_int      black           ; //              54 : non standard   |
    u_int      white           ; //              58 : added for ezfb |
    u_int      gray            ; //              58 :                |
    int        fd              ; //              62 :                |
    u_int      bytes_per_line  ; //              66 :                |
    u_int      palette_size    ; //              70 :                |
    u_char*    data            ; //  null        74 :                |
    u_int*     palette         ; //  null        78 :                |
    u_char*    image           ; //  null        82 :                |
    u_char**   uchar_row       ; //  null        86 :                |
    u_short**  ushort_row      ; //  null        90 :                |
    u_int**    uint_row        ; //  null        94 :                |
    // function pointers
    u_int    (*make_rgb     )(u_char r, u_char g, u_char b);
    u_char   (*r_from_rgb   )(u_int rgb                   );
    u_char   (*g_from_rgb   )(u_int rgb                   );
    u_char   (*b_from_rgb   )(u_int rgb                   );
    int      (*put_pixel    )(struct ezfb_bitmap* bmp, u_int x, u_int y, u_int c);
    int      (*get_pixel    )(struct ezfb_bitmap* bmp, u_int x, u_int y);
    int      (*get_pixel_rgb)(struct ezfb_bitmap* bmp, u_int x, u_int y);
};

//############################################################################
struct ezfb_rectangle
{
    int x1     ;
    int y1     ;
    int x2     ;
    int y2     ;
    int height ;
    int width  ;
};

//############################################################################
//  UTILITY
void   ezfb_dump_rectangle (struct ezfb_rectangle* area, char* message);
void   bmp_dump_header     (struct ezfb_bitmap* bmp, char* file_name, char* message);
//############################################################################








//############################################################################
extern inline int bmp_set_palette_index(struct ezfb_bitmap* bmp, u_int index, u_char r, u_char g, u_char b)
{
    if(     bmp->bpp <= 8
        && (index < (u_int)(0x01 << bmp->bpp))
      )
    {
        bmp->palette[index] = ((r << 16) | (g << 8) | (b));
        return 1;
    }
    else
        return 0;
}

//############################################################################
extern inline u_int bmp_get_palette_index(struct ezfb_bitmap* bmp, u_int index)
{
    if(     bmp->bpp <= 8
        && (index < (u_int)(0x01 << bmp->bpp))
      )
        return bmp->palette[index];
    return 0;
}

//############################################################################
extern inline u_char bmp_get_palette_index_r(struct ezfb_bitmap* bmp, u_int index)
{
    return (bmp_get_palette_index(bmp, index) & 0x00ff0000) >> 16;
}

//############################################################################
extern inline u_char bmp_get_palette_index_g(struct ezfb_bitmap* bmp, u_int index)
{
    return (bmp_get_palette_index(bmp, index) & 0x0000ff00) >> 8;
}

//############################################################################
extern inline u_char bmp_get_palette_index_b(struct ezfb_bitmap* bmp, u_int index)
{
    return bmp_get_palette_index(bmp, index) & 0x000000ff;
}

//############################################################################
extern inline int bmp_is_same_size(struct ezfb_bitmap* a, struct ezfb_bitmap* b)
{
    if(    a->bpp  == b->bpp
        && a->xres == b->xres
        && a->yres == b->yres
      )
        return 1;
    return 0;
}

//############################################################################





//----------------------------------------------------------------------------
//  BITMAP IN MEMORY
void        bmp_create                (struct ezfb_bitmap* bmp, u_int x, u_int y, u_short bpp);
void        bmp_clone_ezfb            (struct ezfb* fb,         struct ezfb_bitmap* bmp, int copy_image);
long double bmp_put_line              (struct ezfb_bitmap* bmp, int x1, int y1, int x2, int y2, u_int c);
void        bmp_put_line_fast         (struct ezfb_bitmap* bmp, int x1, int y1, int x2, int y2, u_int c);
long double bmp_put_ray               (struct ezfb_bitmap* bmp, uint x, uint y, long double r, long double t, uint c);
void        bmp_put_ray_fast          (struct ezfb_bitmap* bmp, uint x, uint y, long double r, long double t, uint c);
void        bmp_put_rectangle         (struct ezfb_bitmap* bmp, int x1, int y1, int x2, int y2, u_int c, int filled);
int         bmp_put_byte_pattern      (struct ezfb_bitmap* bmp, char   byte, int x, int y, int fc, int bc, int dir, int size);
int         bmp_put_ascii             (struct ezfb_bitmap* bmp, char  ascii, int x, int y, int fc, int bc, int dir, int size);
int         bmp_put_string            (struct ezfb_bitmap* bmp, char*   str, int x, int y, int fc, int bc, int dir, int size);
int         bmp_printf                (struct ezfb_bitmap* bmp,              int x, int y, int fc, int bc, int dir, int size, const char* text_format, ...);
void        bmp_dump_palette          (struct ezfb_bitmap* bmp);
void        bmp_dump_image            (struct ezfb_bitmap* bmp);
void        bmp_clear                 (struct ezfb_bitmap* bmp, u_char c);
void        bmp_copy                  (struct ezfb_bitmap* target, struct ezfb_bitmap* original);
void        bmp_flip                  (struct ezfb_bitmap* bmp);
void        bmp_into_bmp              (struct ezfb_bitmap* target, struct ezfb_bitmap* insert);
void        bmp_free                  (struct ezfb_bitmap* bmp);
//----------------------------------------------------------------------------
//  BITMAP MEMORY AND FILE
int         bmp_from_file             (struct ezfb_bitmap* bmp, char* file_name);
int         bmp_save_as               (struct ezfb_bitmap* bmp, char* file_name);
//----------------------------------------------------------------------------
//  FRAME BUFFER OUTPUT AS BMP FILE
int         bmp_save_ezfb_as          (struct ezfb* fb, char* file_name);
int         bmp_save_ezfb_area_as     (struct ezfb* fb, char* file_name, struct ezfb_rectangle* fb_area);
//----------------------------------------------------------------------------
//  FRAME BUFFER AND BMP IN MEMORY
int         bmp_to_ezfb               (struct ezfb* fb, struct ezfb_bitmap* bmp, struct ezfb_rectangle* fb_area, struct ezfb_rectangle* bmp_area);
int         bmp_clone_to_ezfb         (struct ezfb* fb, struct ezfb_bitmap* bmp);
int         bmp_from_ezfb             (struct ezfb* fb, struct ezfb_bitmap* bmp);
int         bmp_palette_to_ezfb       (struct ezfb* fb, struct ezfb_bitmap* bmp);
int         bmp_palette_from_ezfb     (struct ezfb* fb, struct ezfb_bitmap* bmp);
//----------------------------------------------------------------------------
//  BMP FILE AS INPUT TO FRAME BUFFER
int         bmp_file_palette_to_ezfb  (struct ezfb* fb, char* file_name);
int         bmp_file_to_ezfb          (struct ezfb* fb, char* file_name);
int         bmp_file_to_ezfb_at       (struct ezfb* fb, char* file_name, struct ezfb_rectangle* fb_area);
int         bmp_file_area_to_ezfb_at  (struct ezfb* fb, char* file_name, struct ezfb_rectangle* fb_area, struct ezfb_rectangle* bmp_area);
int         bmp_file_to_ezfb_center   (struct ezfb* fb, char* file_name);

//############################################################################
#endif

#ifdef __cplusplus
}
#endif        /* __cplusplus */
//############################################################################
//////////////////////////////////////////////////////////////////////////////
//############################################################################




